<?php

namespace Litecms\Review\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Review\Http\Requests\ReviewsRequest;
use Litecms\Review\Interfaces\ReviewsRepositoryInterface;
use Litecms\Review\Models\Reviews;

/**
 * Resource controller class for reviews.
 */
class ReviewsResourceController extends BaseController
{

    /**
     * Initialize reviews resource controller.
     *
     * @param type ReviewsRepositoryInterface $reviews
     *
     * @return null
     */
    public function __construct(ReviewsRepositoryInterface $reviews)
    {
        parent::__construct();
        $this->repository = $reviews;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Review\Repositories\Criteria\ReviewsResourceCriteria::class);
    }

    /**
     * Display a list of reviews.
     *
     * @return Response
     */
    public function index(ReviewsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Review\Repositories\Presenter\ReviewsPresenter::class)
                ->$function();
        }

        $reviews = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('review::reviews.names'))
            ->view('review::reviews.index', true)
            ->data(compact('reviews', 'view'))
            ->output();
    }

    /**
     * Display reviews.
     *
     * @param Request $request
     * @param Model   $reviews
     *
     * @return Response
     */
    public function show(ReviewsRequest $request, Reviews $reviews)
    {

        if ($reviews->exists) {
            $view = 'review::reviews.show';
        } else {
            $view = 'review::reviews.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('review::reviews.name'))
            ->data(compact('reviews'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new reviews.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReviewsRequest $request)
    {

        $reviews = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('review::reviews.name')) 
            ->view('review::reviews.create', true) 
            ->data(compact('reviews'))
            ->output();
    }

    /**
     * Create new reviews.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReviewsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $reviews                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('review::reviews.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('review/reviews/' . $reviews->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/review/reviews'))
                ->redirect();
        }

    }

    /**
     * Show reviews for editing.
     *
     * @param Request $request
     * @param Model   $reviews
     *
     * @return Response
     */
    public function edit(ReviewsRequest $request, Reviews $reviews)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('review::reviews.name'))
            ->view('review::reviews.edit', true)
            ->data(compact('reviews'))
            ->output();
    }

    /**
     * Update the reviews.
     *
     * @param Request $request
     * @param Model   $reviews
     *
     * @return Response
     */
    public function update(ReviewsRequest $request, Reviews $reviews)
    {
        try {
            $attributes = $request->all();

            $reviews->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('review::reviews.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('review/reviews/' . $reviews->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('review/reviews/' . $reviews->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the reviews.
     *
     * @param Model   $reviews
     *
     * @return Response
     */
    public function destroy(ReviewsRequest $request, Reviews $reviews)
    {
        try {

            $reviews->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('review::reviews.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('review/reviews/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('review/reviews/' . $reviews->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple reviews.
     *
     * @param Model   $reviews
     *
     * @return Response
     */
    public function delete(ReviewsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('review::reviews.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('review/reviews'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/review/reviews'))
                ->redirect();
        }

    }

    /**
     * Restore deleted reviews.
     *
     * @param Model   $reviews
     *
     * @return Response
     */
    public function restore(ReviewsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('review::reviews.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/review/reviews'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/review/reviews/'))
                ->redirect();
        }

    }

}
